#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define min(a,b) ((a < b) ? a : b)
#define max(a,b) ((a > b) ? a : b)
#define nucleotide(c) ((c=='a')?0:((c=='c')?1:((c=='g')?2:((c=='t')?3:-1))))

typedef struct vertex_node_type vertex_node;
struct vertex_node_type
{
   int index;
   vertex_node *next;
};

char iupac [] =
{
   'a', 'c', 'm', 'g', 'r', 's', 'v', 't', 'w', 'y', 'h', 'k', 'd', 'b', 'n'
};

char complement (char ch)
{
   if (ch == 'a')
      return 't';
   if (ch == 'A')
      return 'T';
   if (ch == 'c')
      return 'g';
   if (ch == 'C')
      return 'G';
   if (ch == 'g')
      return 'c';
   if (ch == 'G')
      return 'C';
   if (ch == 't')
      return 'a';
   if (ch == 'T')
      return 'A';
   return ch;
}

void compute_snp (int n, int len_tuple, int *len_str, char **str,
   vertex_node **vertex, int *index)
{
   int i, j, found, len_str2, index2;
   vertex_node *last_vertex, *current_vertex, *current_vertex2;

   for (i = 0; i < 2 * n; i++)
      index [i] = i;
   for (i = 0; i < 2 * n; i++)
   {
      if (index [i] >= 0)
      {
         while (1)
         {
            found = 0;
            last_vertex = NULL;
            current_vertex = vertex [index [i]];
            while (current_vertex != NULL)
            {
               if ( (len_str [current_vertex -> index / 2] < len_tuple)
                 || (current_vertex -> index / 2 == i / 2)
                 || (vertex [current_vertex -> index] == NULL)
                 || (vertex [current_vertex -> index] -> next != NULL)
                 || (vertex [current_vertex -> index - 2 * (current_vertex
                        -> index % 2) + 1] -> next != NULL)
                 || ( (last_vertex != NULL)
                   && ( (len_str [last_vertex -> index / 2]
                            != len_str [current_vertex -> index / 2])
                     || (last_vertex -> index / 2
                            == current_vertex -> index / 2)
                     || (vertex [last_vertex -> index] -> index != vertex
                            [current_vertex -> index] -> index) ) ) )
               {
                  found = 1;
                  break;
               }
               if (last_vertex == NULL)
                  current_vertex2 = vertex [vertex [vertex [index [i]] -> index]
                     -> index - 2 * (vertex [vertex [index [i]] -> index]
                     -> index % 2) + 1];
               current_vertex2 = current_vertex2 -> next;
               last_vertex = current_vertex;
               current_vertex = current_vertex -> next;
            }
            if ( (vertex [index [i]] == NULL)
              || (vertex [index [i]] -> next == NULL) || found
              || (current_vertex2 != NULL) )
               break;
            len_str2 = 0;
            for (j = 0; j < len_str [vertex [index [i]] -> index / 2]; j++)
            {
               found = 0;
               current_vertex = vertex [index [i]];
               while (current_vertex != NULL)
               {
                  if (str [current_vertex -> index][j]
                         != str [vertex [index [i]] -> index][j])
                  {
                     found = 1;
                     break;
                  }
                  current_vertex = current_vertex -> next;
               }
               if (found)
                  len_str2++;
            }
            if (len_str2 * len_tuple > len_str [vertex [index [i]] -> index / 2]
                   + len_tuple - 1)
               break;
            if (vertex [vertex [index [i]] -> index] -> index == i)
            {
               index2 = i;
               do
               {
                  current_vertex = vertex [index2];
                  while (current_vertex != NULL)
                  {
                     index [current_vertex -> index] = current_vertex -> index;
                     current_vertex = current_vertex -> next;
                  }
                  index2 = vertex [index2] -> index;
               }
               while (index2 != i);
               break;
            }
            current_vertex = vertex [index [i]];
            while (current_vertex != NULL)
            {
               index [current_vertex -> index] = - i - 1;
               current_vertex = current_vertex -> next;
            }
            index [vertex [vertex [index [i]] -> index] -> index] = - i - 1;
            index [i] = vertex [vertex [index [i]] -> index] -> index;
         }
      }
   }
}

void add_index (int n, int len_tuple, int *len_str, vertex_node **vertex,
   int *index, int *num_index, int *num_index2, int *index2, int *used,
   int num_index3, int *index3)
{
   int i, len_str2, *len_str3, *len_str4, index4, found;
   vertex_node *current_vertex;

   len_str2 = 0;
   len_str3 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for len_str3\n");
      exit (-1);
   }
   for (i = num_index3 - 1; i >= 0; i--)
      len_str3 [index3 [i] / 2] = 0;
   len_str4 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for len_str4\n");
      exit (-1);
   }
   for (i = num_index3 - 1; i >= 0; i--)
   {
      len_str4 [index3 [i] / 2] = len_str [index3 [i] / 2];
      index4 = index3 [i];
      while (index4 != index [index3 [i]])
      {
         len_str4 [index3 [i] / 2] += len_str [vertex [index4] -> index / 2];
         index4 = vertex [index4] -> index;
      }
      found = 0;
      if (num_index3 > 1)
      {
         current_vertex = vertex [index [index3 [i]]];
         while (current_vertex != NULL)
         {
            if (used [current_vertex -> index])
            {
               if (len_str3 [current_vertex -> index / 2] == 0)
                  len_str3 [current_vertex -> index / 2]
                     = len_str3 [index3 [i] / 2] + len_str4 [index3 [i] / 2];
               else
                  len_str3 [current_vertex -> index / 2] = min (len_str3 [index3
                     [i] / 2] + len_str4 [index3 [i] / 2], len_str3
                     [current_vertex -> index / 2]);
               found = 1;
            }
            current_vertex = current_vertex -> next;
         }
      }
      if (!found)
      {
         if (len_str2 == 0)
            len_str2 = len_str3 [index3 [i] / 2] + len_str4 [index3 [i] / 2];
         else
            len_str2 = min (len_str3 [index3 [i] / 2] + len_str4 [index3 [i]
               / 2], len_str2);
      }
   }
   free (len_str3);
   free (len_str4);
   if (len_str2 >= len_tuple)
   {
      num_index2 [*num_index] = num_index3;
      for (i = 0; i < num_index3; i++)
         index2 [(*num_index) + i] = index3 [num_index3 - i - 1];
      (*num_index) += num_index3;
   }
}

void compute_scc (int n, int len_tuple, int *len_str, vertex_node **vertex,
   int *index, int *num_index, int *num_index2, int *index2, int *used)
{
   int i, *dfnumber, *lowlink, *instack, *stack2, stacktop, stacktop2,
      num_index3, num_index4, found;
   vertex_node **stack, *current_vertex, *current_vertex2, *current_vertex3;

   dfnumber = (int *) malloc (2 * n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for dfnumber\n");
      exit (-1);
   }
   for (i = 0; i < 2 * n; i++)
      dfnumber [i] = -1;
   lowlink = (int *) malloc (2 * n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for lowlink\n");
      exit (-1);
   }
   stack = (vertex_node **) malloc (2 * n * sizeof (vertex_node *));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for stack\n");
      exit (-1);
   }
   instack = (int *) malloc (2 * n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for instack\n");
      exit (-1);
   }
   for (i = 0; i < 2 * n; i++)
      instack [i] = 0;
   stack2 = (int *) malloc (2 * n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for stack2\n");
      exit (-1);
   }
   stacktop = 0;
   stacktop2 = 0;
   current_vertex = (vertex_node *) malloc (sizeof (vertex_node));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for current_vertex\n");
      exit (-1);
   }
   current_vertex -> next = NULL;
   num_index3 = 0;
   for (i = 0; i < 2 * n; i++)
   {
      if ((index [i] >= 0) && (dfnumber [i] < 0))
      {
         current_vertex -> index = i;
         current_vertex2 = current_vertex;
         while (1)
         {
            if ( (current_vertex2 != NULL)
              && (dfnumber [current_vertex2 -> index] < 0) )
            {
               dfnumber [current_vertex2 -> index] = num_index3;
               lowlink [current_vertex2 -> index] = num_index3;
               stack [stacktop] = current_vertex2;
               stacktop++;
               instack [current_vertex2 -> index] = 1;
               stack2 [stacktop2] = current_vertex2 -> index;
               stacktop2++;
               current_vertex2 = vertex [index [current_vertex2 -> index]];
               num_index3++;
            }
            else
            {
               if (current_vertex2 == NULL)
               {
                  stacktop--;
                  if (stacktop == 0)
                     break;
                  current_vertex2 = stack [stacktop];
                  if (lowlink [current_vertex2 -> index]
                         < lowlink [stack [stacktop - 1] -> index])
                     lowlink [stack [stacktop - 1] -> index]
                        = lowlink [current_vertex2 -> index];
               }
               else if (instack [current_vertex2 -> index])
               {
                  if (dfnumber [current_vertex2 -> index]
                         < lowlink [stack [stacktop - 1] -> index])
                     lowlink [stack [stacktop - 1] -> index]
                        = lowlink [current_vertex2 -> index];
               }
               current_vertex2 = current_vertex2 -> next;
            }
            if (current_vertex2 == NULL)
            {
               if (lowlink [stack [stacktop - 1] -> index]
                      == dfnumber [stack [stacktop - 1] -> index])
               {
                  num_index4 = 0;
                  do
                  {
                     stacktop2--;
                     instack [stack2 [stacktop2]] = 0;
                     found = 0;
                     current_vertex3
                        = vertex [index [stack [stacktop - 1] -> index]];
                     while (current_vertex3 != NULL)
                     {
                        if (current_vertex3 -> index
                               == stack [stacktop - 1] -> index)
                        {
                           found = 1;
                           break;
                        }
                        current_vertex3 = current_vertex3 -> next;
                     }
                     if ( ( (stack2 [stacktop2]
                                != stack [stacktop - 1] -> index)
                         || found || (num_index4 > 0) )
                       && (!used [stack2 [stacktop2] / 2]) )
                     {
                        add_index (n, len_tuple, len_str, vertex, index,
                           num_index, num_index2, index2, NULL, 1,
                           &stack2 [stacktop2]);
                        used [stack2 [stacktop2] / 2] = 1;
                        used [index [stack2 [stacktop2]] / 2] = 1;
                        num_index4++;
                     }
                  }
                  while (stack2 [stacktop2] != stack [stacktop - 1] -> index);
               }
            }
         }
      }
   }
   free (dfnumber);
   free (lowlink);
   free (stack);
   free (instack);
   free (stack2);
   free (current_vertex);
}

void compute_node (int n, vertex_node **vertex, int *index, int *used)
{
   int i, index2, index3;
   vertex_node *last_vertex, *current_vertex;

   for (i = 0; i < 2 * n; i++)
   {
      last_vertex = NULL;
      current_vertex = vertex [i];
      while (current_vertex != NULL)
      {
         if ( (index [current_vertex -> index] >= 0)
           && (used [i / 2] || used [current_vertex -> index / 2]) )
         {
            if (last_vertex == NULL)
               vertex [i] = current_vertex -> next;
            else
               last_vertex -> next = current_vertex -> next;
            free (current_vertex);
            if (last_vertex == NULL)
               current_vertex = vertex [i];
            else
               current_vertex = last_vertex -> next;
         }
         else
         {
            last_vertex = current_vertex;
            current_vertex = current_vertex -> next;
         }
      }
   }
   for (i = 0; i < 2 * n; i++)
   {
      if ( (index [i] >= 0) && (!used [i / 2])
        && ( (vertex [i - 2 * (i % 2) + 1] == NULL)
          || (vertex [i - 2 * (i % 2) + 1] -> next != NULL)
          || (vertex [vertex [i - 2 * (i % 2) + 1] -> index - 2 * (vertex [i - 2                 * (i % 2) + 1] -> index % 2) + 1] -> next != NULL)
          || (vertex [i - 2 * (i % 2) + 1] -> index / 2 == i / 2) ) )
      {
         while ( (vertex [index [i]] != NULL)
              && (vertex [index [i]] -> next == NULL)
              && (vertex [vertex [index [i]] -> index - 2 * (vertex [index [i]]
                     -> index % 2) + 1] -> next == NULL)
              && (vertex [index [i]] -> index / 2 != index [i] / 2) )
         {
            index2 = index [vertex [index [i]] -> index];
            index3 = index [i];
            while (index3 != index2)
            {
               current_vertex = vertex [index3];
               while (current_vertex != NULL)
               {
                  index [current_vertex -> index] = - i - 1;
                  current_vertex = current_vertex -> next;
               }
               index3 = vertex [index3] -> index;
            }
            index [i] = index2;
         }
      }
   }
}

void compute_dcc (int n, vertex_node **vertex, int *index, int *used,
   int *used2, int index2)
{
   int *stack, stacktop, index3;
   vertex_node *current_vertex;

   used2 [index2] = 1;
   stack = (int *) malloc (2 * n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for stack\n");
      exit (-1);
   }
   stack [0] = index2;
   stacktop = 1;
   while (stacktop > 0)
   {
      stacktop--;
      index3 = stack [stacktop];
      current_vertex = vertex [index [index3]];
      while (current_vertex != NULL)
      {
         if ( (!used [current_vertex -> index / 2])
           && (!used2 [current_vertex -> index]) )
         {
            used2 [current_vertex -> index] = 1;
            stack [stacktop] = current_vertex -> index;
            stacktop++;
         }
         current_vertex = current_vertex -> next;
      }
      current_vertex = vertex [index3 - 2 * (index3 % 2) + 1];
      while (current_vertex != NULL)
      {
         if ( (!used [current_vertex -> index / 2])
           && (!used2 [index [current_vertex -> index] - 2 * (index
                  [current_vertex -> index] % 2) + 1]) )
         {
            used2 [index [current_vertex -> index] - 2 * (index [current_vertex
               -> index] % 2) + 1] = 1;
            stack [stacktop] = index [current_vertex -> index] - 2 * (index
               [current_vertex -> index] % 2) + 1;
            stacktop++;
         }
         current_vertex = current_vertex -> next;
      }
   }
   free (stack);
}

void compute_index (int n, vertex_node **vertex, int *index, int *used,
   int *num_index, int *index2)
{
   int i, *used2, stacktop;
   vertex_node **stack, *current_vertex, *current_vertex2;

   used2 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for used2\n");
      exit (-1);
   }
   for (i = 0; i < n; i++)
      used2 [i] = 0;
   stack = (vertex_node **) malloc (n * sizeof (vertex_node *));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for stack\n");
      exit (-1);
   }
   stacktop = 0;
   current_vertex = (vertex_node *) malloc (sizeof (vertex_node));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for current_vertex\n");
      exit (-1);
   }
   current_vertex -> next = NULL;
   (*num_index) = 0;
   for (i = 0; i < 2 * n; i++)
   {
      if (used [i] && (!used2 [i / 2]))
      {
         current_vertex -> index = i;
         current_vertex2 = current_vertex;
         while (1)
         {
            if ( (current_vertex2 != NULL) && used [current_vertex2 -> index]
              && (!used2 [current_vertex2 -> index / 2]) )
            {
               used2 [current_vertex2 -> index / 2] = 1;
               used2 [index [current_vertex2 -> index] / 2] = 1;
               stack [stacktop] = current_vertex2;
               stacktop++;
               current_vertex2 = vertex [index [current_vertex2 -> index]];
            }
            else
            {
               if (current_vertex2 == NULL)
               {
                  stacktop--;
                  index2 [*num_index] = stack [stacktop] -> index;
                  (*num_index)++;
                  if (stacktop == 0)
                     break;
                  current_vertex2 = stack [stacktop];
               }
               current_vertex2 = current_vertex2 -> next;
            }
         }
      }
   }
   free (used2);
   free (stack);
   free (current_vertex);
}

void print_index (int n, int len_tuple, int *len_str, char **str,
   vertex_node **vertex, int *index, int num_index, int *index2, int num_read,
   int **read, int *read2)
{
   int i, j, *len_str2, len_str3, len_str4, *num_pos, *num_pos2, *used, index3,
      index4, num_index2, found, pos;
   vertex_node *current_vertex, *current_vertex2;

   len_str2 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for len_str2\n");
      exit (-1);
   }
   num_pos = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for num_pos\n");
      exit (-1);
   }
   num_pos2 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for num_pos2\n");
      exit (-1);
   }
   if (num_index > 1)
   {
      used = (int *) malloc (2 * n * sizeof (int));
      if (errno == ENOMEM)
      {
         printf ("Error: Not enough memory for used\n");
         exit (-1);
      }
      for (i = 0; i < 2 * n; i++)
         used [i] = 0;
      for (i = 0; i < num_index; i++)
         used [index2 [i]] = 1;
   }
   for (i = 0; i < num_index; i++)
   {
      len_str2 [index2 [i] / 2] = len_str [index2 [i] / 2];
      index3 = index2 [i];
      while (index3 != index [index2 [i]])
      {
         len_str2 [index2 [i] / 2] += len_str [vertex [index3] -> index / 2];
         index3 = vertex [index3] -> index;
      }
      num_pos [index2 [i] / 2] = len_tuple - 1;
      num_pos2 [index2 [i] / 2] = 0;
      if (num_index > 1)
      {
         current_vertex = vertex [index2 [i] - 2 * (index2 [i] % 2) + 1];
         while (current_vertex != NULL)
         {
            if (used [index [current_vertex -> index] - 2 * (index
                   [current_vertex -> index] % 2) + 1])
            {
               num_pos [index2 [i] / 2] = min (len_str2 [index [current_vertex
                  -> index] / 2] + num_pos [index [current_vertex -> index]
                  / 2], num_pos [index2 [i] / 2]);
               if (num_pos [index2 [i] / 2] == 0)
                  break;
               num_index2 = 0;
               current_vertex2 = vertex [current_vertex -> index - 2
                  * (current_vertex -> index % 2) + 1];
               while (current_vertex2 != NULL)
               {
                  if (used [current_vertex2 -> index])
                  {
                     num_index2++;
                     if (num_index2 > 1)
                     {
                        num_pos [index2 [i] / 2] = 0;
                        break;
                     }
                  }
                  current_vertex2 = current_vertex2 -> next;
               }
               if (num_pos [index2 [i] / 2] == 0)
                  break;
            }
            current_vertex = current_vertex -> next;
         }
         if (num_pos [index2 [i] / 2] > 0)
         {
            current_vertex = vertex [index2 [i] - 2 * (index2 [i] % 2) + 1];
            while (current_vertex != NULL)
            {
               if (used [index [current_vertex -> index] - 2 * (index
                      [current_vertex -> index] % 2) + 1])
                  num_pos2 [index [current_vertex -> index] / 2]
                     = num_pos [index2 [i] / 2];
               current_vertex = current_vertex -> next;
            }
         }
      }
   }
   for (i = 0; i < num_index; i++)
   {
      printf (">NODE_%d", index2 [i] / 2 + 1);
      if (num_index > 1)
      {
         found = 0;
         for (j = 0; j < num_index; j++)
         {
            current_vertex = vertex [index [index2 [i]]];
            while ( (current_vertex != NULL)
                 && (current_vertex -> index < index2 [j]) )
               current_vertex = current_vertex -> next;
            if ( (current_vertex != NULL)
              && (current_vertex -> index == index2 [j]) )
            {
               if (!found)
                  printf (":");
               else
                  printf (",");
               printf ("%d", index2 [j] / 2 + 1);
               found = 1;
            }
         }
      }
      for (j = 0; j < num_read; j++)
         printf (" %.0f", 1e9 * read [j][index2 [i] / 2] / (len_str2 [index2 [i]
            / 2] + len_tuple - 1) / read2 [j]);
      printf ("\n");
      if (len_str2 [index2 [i] / 2] + num_pos [index2 [i] / 2]
             - num_pos2 [index2 [i] / 2] > 0)
      {
         len_str3 = 0;
         if (num_pos [index2 [i] / 2] > 0)
         {
            pos = len_tuple - num_pos [index2 [i] / 2] - 1;
            index3 = index2 [i];
            while (1)
            {
               for (j = len_str [index3 / 2] - pos - 1; j >= 0; j--)
               {
                  printf ("%c", complement (str [index3 - 2 * (index3 % 2) + 1]
                     [j]));
                  len_str3++;
                  if (len_str3 == min (num_pos [index2 [i] / 2], len_str2
                         [index2 [i] / 2] + num_pos [index2 [i] / 2] - num_pos2
                         [index2 [i] / 2]))
                     break;
               }
               if (len_str3 == min (num_pos [index2 [i] / 2], len_str2 [index2
                      [i] / 2] + num_pos [index2 [i] / 2] - num_pos2 [index2 [i]
                      / 2]))
                  break;
               current_vertex = vertex [index3];
               if (index3 == index [index3])
               {
                  while ( (current_vertex != NULL)
                       && (!used [current_vertex -> index]) )
                     current_vertex = current_vertex -> next;
               }
               if (current_vertex == NULL)
                  break;
               pos = max (0, pos - len_str [index3 / 2]);
               index3 = current_vertex -> index;
            }
            if (len_str3 < min (num_pos [index2 [i] / 2], len_str2 [index2 [i]
                   / 2] + num_pos [index2 [i] / 2] - num_pos2 [index2 [i] / 2]))
            {
               len_str4 = len_str3;
               index3 = index2 [i] - 2 * (index2 [i] % 2) + 1;
               while (1)
               {
                  current_vertex = vertex [index3];
                  if (index3 == index [index3])
                  {
                     while (!used [index [current_vertex -> index] - 2 * (index
                               [current_vertex -> index] % 2) + 1])
                        current_vertex = current_vertex -> next;
                  }
                  len_str4 += len_str [current_vertex -> index / 2];
                  if (len_str4 >= min (num_pos [index2 [i] / 2], len_str2
                         [index2 [i] / 2] + num_pos [index2 [i] / 2] - num_pos2
                         [index2 [i] / 2]))
                     break;
                  index3 = current_vertex -> index;
               }
               pos = len_str4 - min (num_pos [index2 [i] / 2], len_str2 [index2
                  [i] / 2] + num_pos [index2 [i] / 2] - num_pos2 [index2 [i]
                  / 2]);
               index3 = current_vertex -> index - 2 * (current_vertex -> index
                  % 2) + 1;
               while (1)
               {
                  for (j = pos; j < len_str [index3 / 2]; j++)
                  {
                     printf ("%c", str [index3][j]);
                     len_str3++;
                     if (len_str3 == min (num_pos [index2 [i] / 2], len_str2
                            [index2 [i] / 2] + num_pos [index2 [i] / 2]
                            - num_pos2 [index2 [i] / 2]))
                        break;
                  }
                  if (len_str3 == min (num_pos [index2 [i] / 2], len_str2
                         [index2 [i] / 2] + num_pos [index2 [i] / 2] - num_pos2
                         [index2 [i] / 2]))
                     break;
                  current_vertex = vertex [index3];
                  if (index3 == index [index3])
                  {
                     while (!used [current_vertex -> index])
                        current_vertex = current_vertex -> next;
                  }
                  pos = 0;
                  index3 = current_vertex -> index;
               }
            }
         }
         if (len_str3 < len_str2 [index2 [i] / 2] + num_pos [index2 [i] / 2]
                - num_pos2 [index2 [i] / 2])
         {
            for (j = 0; j < len_str [index2 [i] / 2]; j++)
            {
               printf ("%c", str [index2 [i]][j]);
               len_str3++;
               if (len_str3 == len_str2 [index2 [i] / 2] + num_pos [index2 [i]
                      / 2] - num_pos2 [index2 [i] / 2])
                  break;
            }
            if (len_str3 < len_str2 [index2 [i] / 2] + num_pos [index2 [i] / 2]
                   - num_pos2 [index2 [i] / 2])
            {
               index3 = index2 [i];
               while (index3 != index [index2 [i]])
               {
                  for (j = 0; j < len_str [vertex [index3] -> index / 2]; j++)
                  {
                     index4 = 0;
                     current_vertex = vertex [index3];
                     while (current_vertex != NULL)
                     {
                        index4 |= (1 << nucleotide (tolower (str [current_vertex
                           -> index][j])));
                        current_vertex = current_vertex -> next;
                     }
                     printf ("%c", toupper (iupac [index4 - 1]));
                     len_str3++;
                     if (len_str3 == len_str2 [index2 [i] / 2] + num_pos [index2
                            [i] / 2] - num_pos2 [index2 [i] / 2])
                        break;
                  }
                  if (len_str3 == len_str2 [index2 [i] / 2] + num_pos [index2
                         [i] / 2] - num_pos2 [index2 [i] / 2])
                     break;
                  index3 = vertex [index3] -> index;
               }
            }
         }
         printf ("\n");
      }
   }
   free (len_str2);
   free (num_pos);
   free (num_pos2);
   if (num_index > 1)
      free (used);
   printf ("\n");
}

main (int argc, char **argv)
{
   FILE *file;
   int i, j, k, n, ch, *read_end, found, len_tuple, *len_str, *index,
      index1 [2], index2 [2], *index3, *index4, num_index, *num_index2,
      num_index3, *used, *used2, **read, *read2;
   char buffer [1000000], **str;
   vertex_node **vertex, ***vertex2, *last_vertex, *current_vertex, *new_vertex;

   read_end = (int *) malloc (argc * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for read_end\n");
      exit (-1);
   }
   read_end [0] = 0;
   for (i = 1; i < argc; i++)
   {
      read_end [i] = read_end [i - 1];
      file = fopen (argv [i], "r");
      if (file == NULL)
      {
         printf ("Error: Cannot open file %s\n", argv [i]);
         exit (-1);
      }
      found = 0;
      do
      {
         ch = getc (file);
         if (found < 0)
         {
            if (ch == '\n')
               found++;
         }
         else if (found == 0)
         {
            if ((ch == '>') || (ch == '@'))
               found++;
         }
         else if (found == 1)
         {
            if (ch == '\n')
            {
               read_end [i]++;
               found++;
            }
         }
         else if (ch == '>')
            found = 1;
         else if (ch == '+')
            found = -2;
      }
      while (ch != EOF);
      if (fclose (file) == EOF)
      {
         printf ("Error: Cannot close file %s\n", argv [i]);
         exit (-1);
      }
   }
   gets (buffer);
   sscanf (buffer, "%d %*d %d", &n, &len_tuple);
   len_str = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for len_str\n");
      exit (-1);
   }
   str = (char **) malloc (2 * n * sizeof (char *));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for str\n");
      exit (-1);
   }
   for (i = 0; i < n; i++)
   {
      gets (buffer);
      sscanf (buffer, "%*s %*d %d", &len_str [i]);
      for (j = 0; j < 2; j++)
      {
         str [2 * i + j] = (char *) malloc ((len_str [i] + 1) * sizeof (char));
         if (errno == ENOMEM)
         {
            printf ("Error: Not enough memory for str\n");
            exit (-1);
         }
         gets (str [2 * i + j]);
      }
   }
   vertex = (vertex_node **) malloc (2 * n * sizeof (vertex_node *));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for vertex\n");
      exit (-1);
   }
   for (i = 0; i < 2 * n; i++)
      vertex [i] = NULL;
   gets (buffer);
   for (i = 0; i < 2 * n; i++)
   {
      do
      {
         if (strncmp (buffer, "ARC", strlen ("ARC")))
            break;
         sscanf (buffer, "%*s %d %d", &index1 [0], &index1 [1]);
         if (2 * (abs (index1 [0]) - 1) + (index1 [0] < 0) > i)
            break;
         for (j = 0; j < 2; j++)
         {
            for (k = 0; k < 2; k++)
               index2 [k]
                  = 2 * (abs (index1 [k]) - 1) + ((1 - 2 * j) * index1 [k] < 0);
            last_vertex = NULL;
            current_vertex = vertex [index2 [j]];
            while ( (current_vertex != NULL)
                 && (current_vertex -> index < index2 [1 - j]) )
            {
               last_vertex = current_vertex;
               current_vertex = current_vertex -> next;
            }
            if ( (current_vertex == NULL)
              || (current_vertex -> index > index2 [1 - j]) )
            {
               new_vertex = (vertex_node *) malloc (sizeof (vertex_node));
               if (errno == ENOMEM)
               {
                  printf ("Error: Not enough memory for new_vertex\n");
                  exit (-1);
               }
               new_vertex -> index = index2 [1 - j];
               new_vertex -> next = current_vertex;
               if (last_vertex == NULL)
                  vertex [index2 [j]] = new_vertex;
               else
                  last_vertex -> next = new_vertex;
            }
         }
      }
      while (gets (buffer) != NULL);
   }
   index = (int *) malloc (2 * n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for index\n");
      exit (-1);
   }
   compute_snp (n, len_tuple, len_str, str, vertex, index);
   num_index = 0;
   num_index2 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for num_index2\n");
      exit (-1);
   }
   index3 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for index3\n");
      exit (-1);
   }
   used = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for used\n");
      exit (-1);
   }
   for (i = 0; i < n; i++)
      used [i] = 0;
   compute_scc (n, len_tuple, len_str, vertex, index, &num_index, num_index2,
      index3, used);
   compute_node (n, vertex, index, used);
   used2 = (int *) malloc (2 * n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for used2\n");
      exit (-1);
   }
   index4 = (int *) malloc (n * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for index4\n");
      exit (-1);
   }
   for (i = 0; i < 2 * n; i++)
   {
      if ((index [i] >= 0) && (!used [i / 2]))
      {
         for (j = 0; j < 2 * n; j++)
            used2 [j] = 0;
         compute_dcc (n, vertex, index, used, used2, i);
         found = 0;
         for (j = 0; j < 2 * n; j++)
         {
            if (used2 [j])
            {
               if (!used [j / 2])
               {
                  used [j / 2] = 1;
                  used [index [j] / 2] = 1;
               }
               else
                  found = 1;
            }
         }
         compute_index (n, vertex, index, used2, &num_index3, index4);
         if (found)
         {
            for (j = num_index3 - 1; j >= 0; j--)
               add_index (n, len_tuple, len_str, vertex, index, &num_index,
                  num_index2, index3, NULL, 1, &index4 [j]);
         }
         else
            add_index (n, len_tuple, len_str, vertex, index, &num_index,
               num_index2, index3, used2, num_index3, index4);
      }
   }
   free (used2);
   free (index4);
   for (i = 0; i < n; i++)
      used [i] = 0;
   for (i = 0; i < num_index; i++)
      used [index3 [i] / 2] = 1;
   vertex2 = (vertex_node ***) malloc ((argc - 1) * sizeof (vertex_node **));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for vertex2\n");
      exit (-1);
   }
   for (i = 0; i < argc - 1; i++)
   {
      vertex2 [i] = (vertex_node **)
         malloc ((read_end [i + 1] - read_end [i]) * sizeof (vertex_node *));
      if (errno == ENOMEM)
      {
         printf ("Error: Not enough memory for vertex2\n");
         exit (-1);
      }
      for (j = 0; j < read_end [i + 1] - read_end [i]; j++)
         vertex2 [i][j] = NULL;
   }
   if (!strncmp (buffer, "SEQ", strlen ("SEQ")))
   {
      do
      {
         if (!strncmp (buffer, "NR", strlen ("NR")))
            break;
         if (!strncmp (buffer, "SEQ", strlen ("SEQ")))
         {
            sscanf (buffer, "%*s %d", &index1 [0]);
            index2 [0] = 0;
            while ( (index2 [0] < argc - 1)
                 && (read_end [index2 [0] + 1] < index1 [0]) )
               index2 [0]++;
         }
         else if (index2 [0] < argc - 1)
         {
            sscanf (buffer, "%d", &index1 [1]);
            index1 [1] = 2 * (abs (index1 [1]) - 1);
            if (index [index1 [1]] < 0)
               index1 [1] = - index [index1 [1]] - 1;
            if (!used [index1 [1] / 2])
               index1 [1] = index [index1 [1]];
            if (used [index1 [1] / 2])
            {
               last_vertex = NULL;
               current_vertex = vertex2 [index2 [0]][index1 [0] - read_end
                  [index2 [0]] - 1];
               while ( (current_vertex != NULL)
                    && (current_vertex -> index < index1 [1] / 2) )
               {
                  last_vertex = current_vertex;
                  current_vertex = current_vertex -> next;
               }
               if ( (current_vertex == NULL)
                 || (current_vertex -> index > index1 [1] / 2) )
               {
                  new_vertex = (vertex_node *) malloc (sizeof (vertex_node));
                  if (errno == ENOMEM)
                  {
                     printf ("Error: Not enough memory for new_vertex\n");
                     exit (-1);
                  }
                  new_vertex -> index = index1 [1] / 2;
                  new_vertex -> next = current_vertex;
                  if (last_vertex == NULL)
                     vertex2 [index2 [0]][index1 [0] - read_end [index2 [0]]
                        - 1] = new_vertex;
                  else
                     last_vertex -> next = new_vertex;
               }
            }
         }
      }
      while (gets (buffer) != NULL);
   }
   do
   {
      if (strncmp (buffer, "NR", strlen ("NR")))
         break;
      sscanf (buffer, "%*s %d %d", &index1 [0], &num_index3);
      index1 [0] = 2 * (abs (index1 [0]) - 1);
      if (index [index1 [0]] < 0)
         index1 [0] = - index [index1 [0]] - 1;
      if (!used [index1 [0] / 2])
         index1 [0] = index [index1 [0]];
      index2 [0] = 0;
      for (i = 0; i < num_index3; i++)
      {
         gets (buffer);
         if (used [index1 [0] / 2])
         {
            sscanf (buffer, "%d", &index1 [1]);
            while ( (index2 [0] < argc - 1)
                 && (read_end [index2 [0] + 1] < index1 [1]) )
               index2 [0]++;
            if (index2 [0] < argc - 1)
            {
               last_vertex = NULL;
               current_vertex = vertex2 [index2 [0]][index1 [1] - read_end
                  [index2 [0]] - 1];
               while ( (current_vertex != NULL)
                    && (current_vertex -> index < index1 [0] / 2) )
               {
                  last_vertex = current_vertex;
                  current_vertex = current_vertex -> next;
               }
               if ( (current_vertex == NULL)
                 || (current_vertex -> index > index1 [0] / 2) )
               {
                  new_vertex = (vertex_node *) malloc (sizeof (vertex_node));
                  if (errno == ENOMEM)
                  {
                     printf ("Error: Not enough memory for new_vertex\n");
                     exit (-1);
                  }
                  new_vertex -> index = index1 [0] / 2;
                  new_vertex -> next = current_vertex;
                  if (last_vertex == NULL)
                     vertex2 [index2 [0]][index1 [1] - read_end [index2 [0]]
                        - 1] = new_vertex;
                  else
                     last_vertex -> next = new_vertex;
               }
            }
         }
      }
   }
   while (gets (buffer) != NULL);
   free (used);
   read = (int **) malloc ((argc - 1) * sizeof (int *));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for read\n");
      exit (-1);
   }
   read2 = (int *) malloc ((argc - 1) * sizeof (int));
   if (errno == ENOMEM)
   {
      printf ("Error: Not enough memory for read2\n");
      exit (-1);
   }
   for (i = 0; i < argc - 1; i++)
   {
      read [i] = (int *) malloc (n * sizeof (int));
      if (errno == ENOMEM)
      {
         printf ("Error: Not enough memory for read\n");
         exit (-1);
      }
      for (j = 0; j < n; j++)
         read [i][j] = 0;
      read2 [i] = 0;
      for (j = 0; j < read_end [i + 1] - read_end [i]; j++)
      {
         if (vertex2 [i][j] != NULL)
         {
            while (vertex2 [i][j] != NULL)
            {
               current_vertex = vertex2 [i][j];
               vertex2 [i][j] = vertex2 [i][j] -> next;
               read [i][current_vertex -> index]++;
               free (current_vertex);
            }
            read2 [i]++;
         }
      }
      free (vertex2 [i]);
   }
   free (vertex2);
   free (read_end);
   for (i = 0; i < num_index; i += num_index2 [i])
      print_index (n, len_tuple, len_str, str, vertex, index, num_index2 [i],
         &index3 [i], argc - 1, read, read2);
   free (len_str);
   for (i = 0; i < 2 * n; i++)
      free (str [i]);
   free (str);
   for (i = 0; i < 2 * n; i++)
   {
      while (vertex [i] != NULL)
      {
         current_vertex = vertex [i];
         vertex [i] = vertex [i] -> next;
         free (current_vertex);
      }
   }
   free (vertex);
   free (index);
   free (num_index2);
   free (index3);
   for (i = 0; i < argc - 1; i++)
      free (read [i]);
   free (read);
   free (read2);
}
